using System.Collections.Generic;
using System.Net.WebSockets;
using System.Text.Json;
using System.Threading;
using BubbleSocketCore.Library;
using BubbleSocketCore.Websocket;
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.Logging;

namespace BubbleSocketCore.Simulation
{
	public class ConnectSimulationActionType : ISimulationAction
	{
		private readonly IConfiguration Configuration;
		private readonly ILogger Logger;
		private readonly JsonFileManager JsonFileManager;

		public ConnectSimulationActionType(IConfiguration configuration, ILoggerFactory loggerFactory) {
			Configuration = configuration;
			Logger = loggerFactory.CreateLogger("ConnectSimulationActionType");
			JsonFileManager = new JsonFileManager(Configuration);
		}
		public object ParseMessageData(string rawString)
		{
			return JsonSerializer.Deserialize<List<JsonElement>>(rawString);
		}
		public async void Run(WebSocket socket, object transmittedData)
		{
			//TODO: extract info from connect request
			var connectionData = transmittedData as List<JsonElement>;
			var token = connectionData[0].ToString();

			var username = new JwtTokenHandler(Configuration).GetTokenName(token);

			var accountInformation = JsonFileManager.ReadAccountFile(username);
			var accountId = accountInformation.GetProperty("AccountId").ToString();
			var characterList = JsonFileManager.GetCharacterList(accountId);

			var connection = ConnectionManager.GetInstance().GetConnection(socket.GetHashCode());
			// connection.SetEntityId(entityId);

			//TODO: broadcast connect info to all relevant connections

			// List<object> playerConnectPayload = new List<object>();
			// playerConnectPayload.Add(entityId);
			// playerConnectPayload.Add(Simulation.latestFrame);
			// playerConnectPayload.Add(newEntity.Serialize());
			// ConnectionManager.GetInstance().GetActive().ForEach(connection =>
			// {
			// 	connection.SendAsync(SimulationActionType.PlayerConnect, playerConnectPayload);
			// });


			//TODO: return initial state response if successful

			Dictionary<string, object> payload = new Dictionary<string, object>();
			payload.Add("sessionId", socket.GetHashCode());
			payload.Add("characters", characterList);
			await connection.SendAsync(SimulationActionType.Connect, payload, CancellationToken.None);
		}
	}
}